/**
*  OpenKM, Open Document Management System (http://www.openkm.com)
 *  Copyright (c) 2006-2014  Paco Avila & Josep Llort
 *
 *  No bytes were intentionally harmed during the development of this application.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package com.openkm.extension.frontend.client.widget.customer.sample;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.openkm.extension.frontend.client.bean.GWTExtendedSecurity;
import com.openkm.extension.frontend.client.util.OKMBundleResources;
import com.openkm.frontend.client.bean.GWTDocument;
import com.openkm.frontend.client.bean.GWTPermission;
import com.openkm.frontend.client.bean.GWTUser;
import com.openkm.frontend.client.bean.ToolBarOption;
import com.openkm.frontend.client.extension.comunicator.GeneralComunicator;
import com.openkm.frontend.client.extension.comunicator.ToolBarComunicator;
import com.openkm.frontend.client.extension.event.HasToolBarEvent;
import com.openkm.frontend.client.extension.event.HasToolBarEvent.ToolBarEventConstant;
import com.openkm.frontend.client.extension.event.handler.ToolBarHandlerExtension;
import com.openkm.frontend.client.extension.widget.tabdocument.TabDocumentExtension;
import com.openkm.frontend.client.service.OKMAuthService;
import com.openkm.frontend.client.service.OKMAuthServiceAsync;
import com.openkm.frontend.client.service.OKMDatabaseMetadataService;
import com.openkm.frontend.client.service.OKMDatabaseMetadataServiceAsync;
import com.openkm.frontend.client.service.OKMRepositoryService;
import com.openkm.frontend.client.service.OKMRepositoryServiceAsync;

/**
 * ExtendedSecurityTabDocument
 * 
 * @author jllort
 *
 */
public class ExtendedSecurityTabDocument extends TabDocumentExtension implements ToolBarHandlerExtension {
	private final OKMAuthServiceAsync authService = (OKMAuthServiceAsync) GWT.create(OKMAuthService.class);
	private final OKMDatabaseMetadataServiceAsync metadataService = (OKMDatabaseMetadataServiceAsync) GWT.create(OKMDatabaseMetadataService.class);
	private final OKMRepositoryServiceAsync repositoryService = (OKMRepositoryServiceAsync) GWT.create(OKMRepositoryService.class);
	
	private ScrollPanel scrollPanel;
	private FlexTable table;
	private List<GWTUser> allUserList;
	private List<String> allRoleList;
	private List<String> restrictedUserList;
	private List<String> restrictedRoleList;
	private List<String> restrictedParentUserList;
	private List<String> restrictedParentRoleList;
	private GWTDocument doc;
	private ListBox availableUsers;
	private ListBox availableRoles;
	private String user;
	private List<String> userRoles;
	private Button addUserButton;
	private Button addRoleButton;
	private VerticalPanel vUserPanel;
	private VerticalPanel vRolePanel;
	private String uuid = ""; 
	private boolean defaultDownloadOption;
	
	/**
	 * TabSecurityDownload
	 */
	public ExtendedSecurityTabDocument() {
		availableUsers = new ListBox();
		availableUsers.setStyleName("okm-Input");
		availableRoles = new ListBox();
		availableRoles.setStyleName("okm-Input");
		addUserButton = new Button(GeneralComunicator.i18n("button.add"));
		addUserButton.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				if (availableUsers.getItemCount()>0 && availableUsers.getSelectedIndex()>0) {
					final String name = availableUsers.getValue(availableUsers.getSelectedIndex());
					final GWTExtendedSecurity security = new GWTExtendedSecurity();
					security.setUuid(doc.getUuid());
					security.setName(name);
					security.setType(GWTExtendedSecurity.TYPE_USER);
					security.setRealTable(GWTExtendedSecurity.MV_TABLE_NAME);
					sample.get().status.setAddGrant();
					metadataService.createValue(security.restoreToMap(), new AsyncCallback<Double>() {
						@Override
						public void onSuccess(Double result) {
							security.setRealId(result);
							security.setRealTable(GWTExtendedSecurity.MV_TABLE_NAME);
							vUserPanel.add(new SecurityElement(security, vUserPanel, true));
							availableUsers.removeItem(availableUsers.getSelectedIndex());
							restrictedUserList.add(name);
							evaluateExtendedSecurity();
							sample.get().status.unsetAddGrant();
						}
						@Override
						public void onFailure(Throwable caught) {
							GeneralComunicator.showError("createValue: extended security addRole", caught);
							sample.get().status.unsetAddGrant();
						}
					});
				}
			}
		});
		addUserButton.setStyleName("okm-AddButton");
		addRoleButton = new Button(GeneralComunicator.i18n("button.add"));
		addRoleButton.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				if (availableRoles.getItemCount()>0 && availableRoles.getSelectedIndex()>0) {
					final String name = availableRoles.getValue(availableRoles.getSelectedIndex());
					final GWTExtendedSecurity security = new GWTExtendedSecurity();
					security.setUuid(doc.getUuid());
					security.setName(name);
					security.setType(GWTExtendedSecurity.TYPE_ROLE);
					security.setRealTable(GWTExtendedSecurity.MV_TABLE_NAME);
					sample.get().status.setAddGrant();
					metadataService.createValue(security.restoreToMap(), new AsyncCallback<Double>() {
						@Override
						public void onSuccess(Double result) {
							security.setRealId(result);
							vRolePanel.add(new SecurityElement(security, vRolePanel, false));
							availableRoles.removeItem(availableRoles.getSelectedIndex());
							restrictedRoleList.add(name);
							evaluateExtendedSecurity();
							sample.get().status.unsetAddGrant();
						}
						@Override
						public void onFailure(Throwable caught) {
							GeneralComunicator.showError("createValue: extended security addRole", caught);
							sample.get().status.unsetAddGrant();
						}
					});
				}
			}
		});
		addRoleButton.setStyleName("okm-AddButton");
		
		vUserPanel = new VerticalPanel();
		vRolePanel = new VerticalPanel();
		
		table = new FlexTable();
		table.setCellPadding(0);
		table.setCellSpacing(2);
		// Row 0
		table.setHTML(0, 0, "<b>" +GeneralComunicator.i18nExtension("sample.tab.security.explain") + "</b>");
		table.getFlexCellFormatter().setColSpan(0, 0, 4);
		// Row 1
		table.setHTML(1, 0, "<b>" +GeneralComunicator.i18nExtension("sample.tab.security.users") + "</b>");		
		HorizontalPanel hUserPanel = new HorizontalPanel();
		hUserPanel.add(new HTML("&nbsp;"));
		hUserPanel.add(availableUsers);
		hUserPanel.add(new HTML("&nbsp;"));
		hUserPanel.add(addUserButton);
		hUserPanel.add(new HTML("&nbsp;&nbsp;&nbsp;"));
		table.setWidget(1, 1, hUserPanel);
		table.setHTML(1, 2, "<b>" +GeneralComunicator.i18nExtension("sample.tab.security.roles") + "</b>");
		HorizontalPanel hRolePanel = new HorizontalPanel();
		hRolePanel.add(new HTML("&nbsp;"));
		hRolePanel.add(availableRoles);
		hRolePanel.add(new HTML("&nbsp;"));
		hRolePanel.add(addRoleButton);
		table.setWidget(1, 3, hRolePanel);		
		// Row 2
		table.setWidget(2, 0, vUserPanel);
		table.setWidget(2, 2, vRolePanel);
		table.getFlexCellFormatter().setColSpan(2, 0, 2);
		table.getFlexCellFormatter().setColSpan(2, 2, 2);
		table.getCellFormatter().setVerticalAlignment(2, 0, HasAlignment.ALIGN_TOP);
		table.getCellFormatter().setVerticalAlignment(2, 2, HasAlignment.ALIGN_TOP);
		
		allUserList = new ArrayList<GWTUser>();
		allRoleList = new ArrayList<String>();
		restrictedUserList = new ArrayList<String>();
		restrictedRoleList = new ArrayList<String>();
		restrictedParentUserList = new ArrayList<String>();
		restrictedParentRoleList = new ArrayList<String>();
		// Get all users
		authService.getAllUsers(new AsyncCallback<List<GWTUser>>() {
			@Override
			public void onSuccess(List<GWTUser> result) {
				allUserList = result;
				// Get al roles
				authService.getAllRoles(new AsyncCallback<List<String>>() {
					@Override
					public void onSuccess(List<String> result) {
						allRoleList = result;
					}
					@Override
					public void onFailure(Throwable caught) {
						GeneralComunicator.showError("getAllRoles", caught);
					}
				});
			}
			@Override
			public void onFailure(Throwable caught) {
				GeneralComunicator.showError("getAllUsers", caught);
			}
		});
		
		scrollPanel = new ScrollPanel(table);
		initWidget(scrollPanel);
	}
	
	@Override
	public void onChange(ToolBarEventConstant event) {
		if (event.equals(HasToolBarEvent.EXECUTE_CHECK_DOCUMENT_PERMISSION)) {
			sample.get().setTabDocumentSelected();
			// Always disabling download button
			ToolBarOption toolBarOption = ToolBarComunicator.getToolBarOption();
			// Save default tab options
			doc = (GWTDocument) ToolBarComunicator.getActualNode(); // Is not sure in TabDocument setDocument be yet executed !
			if (!uuid.equals(doc.getUuid())) {
				defaultDownloadOption = ToolBarComunicator.getToolBarOption().downloadOption;
			}
			toolBarOption.downloadOption = false;
			toolBarOption.downloadPdfOption = false;
			ToolBarComunicator.setToolBarOption(toolBarOption);
			ToolBarComunicator.evaluateShowIcons();
			
			// Enable / disable change security
			if (((doc.getPermissions() & GWTPermission.SECURITY) == GWTPermission.SECURITY)) {
				addUserButton.setEnabled(true);
				addRoleButton.setEnabled(true);
			} else {
				addUserButton.setEnabled(false);
				addRoleButton.setEnabled(false);
			}
			user = GeneralComunicator.getUser();
			userRoles = GeneralComunicator.getWorkspace().getRoleList();
			clearPanels();
			getExtendedSecurity(); 
		}
	}
	
	/**
	 * clearPanels
	 */
	private void clearPanels() {
		while (vUserPanel.getWidgetCount()>0) {
			vUserPanel.remove(0);
		}
		while (vRolePanel.getWidgetCount()>0) {
			vRolePanel.remove(0);
		}
	}
	
	/**
	 * getExtendedSecurity
	 */
	private void getExtendedSecurity() {
		sample.get().status.setGetGrants();
		// Getting folder security restrictions
		repositoryService.getUUIDByPath(doc.getParentPath(), new AsyncCallback<String>() {
			@Override
			public void onSuccess(String result) {
				String filter = "$" + GWTExtendedSecurity.MV_COLUMN_NAME_UUID + "='" + result + "'";
				String order = "";
				metadataService.executeValueQuery(GWTExtendedSecurity.MV_TABLE_NAME, filter, order, new AsyncCallback<List<Map<String,String>>>() {
					@Override
					public void onSuccess(List<Map<String, String>> result) {
						restrictedParentUserList = new ArrayList<String>();
						restrictedParentRoleList = new ArrayList<String>();
						for (Map<String, String> map : result) {
							GWTExtendedSecurity security = new GWTExtendedSecurity();
							security.loadFromMap(map);
							if (security.getType().equals(GWTExtendedSecurity.TYPE_USER)) {
								restrictedParentUserList.add(security.getName());
							} else if (security.getType().equals(GWTExtendedSecurity.TYPE_ROLE)) {
								restrictedParentRoleList.add(security.getName());
							}
						}
						// Getting document security
						String filter = "$" + GWTExtendedSecurity.MV_COLUMN_NAME_UUID + "='" + doc.getUuid() + "'";
						String order = "";
						metadataService.executeValueQuery(GWTExtendedSecurity.MV_TABLE_NAME, filter, order, new AsyncCallback<List<Map<String,String>>>() {
							@Override
							public void onSuccess(List<Map<String, String>> result) {
								restrictedUserList = new ArrayList<String>();
								restrictedRoleList = new ArrayList<String>();
								for (Map<String, String> map : result) {
									GWTExtendedSecurity security = new GWTExtendedSecurity();
									security.loadFromMap(map);
									if (security.getType().equals(GWTExtendedSecurity.TYPE_USER)) {
										restrictedUserList.add(security.getName());
										vUserPanel.add(new SecurityElement(security, vUserPanel, true));
									} else if (security.getType().equals(GWTExtendedSecurity.TYPE_ROLE)) {
										restrictedRoleList.add(security.getName());
										vRolePanel.add(new SecurityElement(security, vRolePanel, false));
									}
								}
								availableUsers.clear();
								availableRoles.clear();
								availableUsers.addItem("",""); // First item always is empty
								availableRoles.addItem("","");
								for (GWTUser user : allUserList) {
									if (!restrictedUserList.contains(user.getId())) {
										availableUsers.addItem(user.getUsername(), user.getId());
									}
								}
								for (String role : allRoleList) {
									if (!restrictedRoleList.contains(role)) {
										availableRoles.addItem(role, role);
									}
								}
								evaluateExtendedSecurity();
								sample.get().status.unsetGetGrants();
							}
							
							@Override
							public void onFailure(Throwable caught) {
								GeneralComunicator.showError("executeValueQuery: findRestrictions", caught);
								sample.get().status.unsetGetGrants();
							}
						});
					}
					@Override
					public void onFailure(Throwable caught) {
						GeneralComunicator.showError("executeValueQuery: findRestrictions", caught);
						sample.get().status.unsetGetGrants();
					}
				});
			}
			
			@Override
			public void onFailure(Throwable caught) {
				GeneralComunicator.showError("executeValueQuery: findRestrictions(parent)", caught);
				sample.get().status.unsetGetGrants();
			}
		});
	}
	
	/**
	 * evaluateExtendedSecurity
	 */
	private void evaluateExtendedSecurity() {
		boolean found = false;
		if (restrictedUserList.contains(user)) {
			found = true;
		} else if (restrictedParentUserList.contains(user)) {
			found = true;
		} else {
			for (String role :userRoles ) {
				if (restrictedRoleList.contains(role)) {
					found = true;
					break;
				} else if (restrictedParentRoleList.contains(role)) {
					found = true;
					break;
				}
			}
		}
		// Restore default toolbar option if not found otherside disables
		if (found) {
			ToolBarOption toolBarOption = ToolBarComunicator.getToolBarOption();
			toolBarOption.downloadOption = false;
			toolBarOption.downloadPdfOption = false;
			ToolBarComunicator.setToolBarOption(toolBarOption);
			ToolBarComunicator.evaluateShowIcons();
		} else {
			ToolBarOption toolBarOption = ToolBarComunicator.getToolBarOption();
			toolBarOption.downloadOption = defaultDownloadOption;
			toolBarOption.downloadPdfOption = defaultDownloadOption;
			ToolBarComunicator.setToolBarOption(toolBarOption);
			ToolBarComunicator.evaluateShowIcons();
		}
	}
	
	/**
	 * langRefresh
	 */
	public void langRefresh() {
		table.setHTML(0, 0, "<b>" +GeneralComunicator.i18nExtension("sample.tab.security.explain") + "</b>");
		table.setHTML(1, 0, "<b>" +GeneralComunicator.i18nExtension("sample.tab.security.users") + "</b>");
		table.setHTML(1, 2, "<b>" +GeneralComunicator.i18nExtension("sample.tab.security.roles") + "</b>");
		addUserButton.setHTML(GeneralComunicator.i18n("button.add"));
		addRoleButton.setHTML(GeneralComunicator.i18n("button.add"));
	}

	@Override
	public String getTabText() {
		return GeneralComunicator.i18nExtension("sample.tab.security");
	}	
	
	/**
	 * SecurityElement
	 * 
	 * @author jllort
	 *
	 */
	class SecurityElement extends HorizontalPanel {
		SecurityElement securityElement;
		public SecurityElement(final GWTExtendedSecurity security, final VerticalPanel vPanel, final boolean isUser) {
			super();
			securityElement = this;
			add(new HTML(security.getName() + "&nbsp;"));
			if (((doc.getPermissions() & GWTPermission.SECURITY) == GWTPermission.SECURITY)) {
				Image delete = new Image(OKMBundleResources.INSTANCE.delete());
				delete.addClickHandler(new ClickHandler() {
					@Override
					public void onClick(ClickEvent event) {
						vPanel.remove(securityElement);
						sample.get().status.setRevokeGrant();
						metadataService.deleteValue(security.restoreToMap(), new AsyncCallback<Object>() {
							@Override
							public void onSuccess(Object result) {
								if (isUser) {
									restrictedUserList.remove(security.getName());
									availableUsers.addItem(security.getName(), security.getName());
								} else {
									restrictedRoleList.remove(security.getName());
									availableRoles.addItem(security.getName(), security.getName());
								}
								evaluateExtendedSecurity();
								sample.get().status.unsetRevokeGrant();
							}
							@Override
							public void onFailure(Throwable caught) {
								GeneralComunicator.showError("deleteValue: extended security", caught);
							}
						});
					}
				});
				delete.setStyleName("okm-Hyperlink");
				add(delete);
			}
		}
	}
}
