/**
 *  OpenKM, Open Document Management System (http://www.openkm.com)
 *  Copyright (c) 2006-2014  Paco Avila & Josep Llort
 *
 *  No bytes were intentionally harmed during the development of this application.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package com.openkm.extension.frontend.client.widget.customer.sample;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.Widget;
import com.openkm.extension.frontend.client.util.OKMBundleResources;
import com.openkm.frontend.client.extension.comunicator.GeneralComunicator;

/**
 * Status
 * 
 * @author jllort
 *
 */
public class Status extends PopupPanel {
	private HorizontalPanel hPanel;
	private HTML msg;
	private HTML space;
	private Image image;
	
	private boolean flag_getGrants = false;
	private boolean flag_addGrant = false;
	private boolean flag_revokeGrant = false;

	/**
	 * The status
	 */
	public Status() {
		super(false,true);
		hPanel = new HorizontalPanel();
		image = new Image(OKMBundleResources.INSTANCE.indicator());
		msg = new HTML("");
		space = new HTML("");
		
		hPanel.add(image);
		hPanel.add(msg);
		hPanel.add(space);
		
		hPanel.setCellVerticalAlignment(image, HasAlignment.ALIGN_MIDDLE);
		hPanel.setCellVerticalAlignment(msg, HasAlignment.ALIGN_MIDDLE);
		hPanel.setCellHorizontalAlignment(image, HasAlignment.ALIGN_CENTER);
		hPanel.setCellWidth(image, "30px");
		hPanel.setCellWidth(space, "7px");
		
		hPanel.setHeight("25px");
		
		msg.setStyleName("okm-NoWrap");
		
		super.hide();
		setWidget(hPanel);
	}
	
	/**
	 * Refreshing the panel
	 */
	public void refresh() {
		if (flag_addGrant || flag_getGrants || flag_revokeGrant || flag_addGrant) {
			Widget tab = sample.get().getWidgetTab();
			int left = tab.getAbsoluteLeft() + (tab.getOffsetWidth() - 200) / 2;
			int top = tab.getAbsoluteTop() + (tab.getOffsetHeight() - 40) / 2;

			if (left > 0 && top > 0) {
				setPopupPosition(left, top);
				super.show();
			}
		} else {
			super.hide();
		}
	}
	
	/**
	 * Sets add grant flag
	 */
	public void setAddGrant() {
		msg.setHTML(GeneralComunicator.i18nExtension("sample.tab.add.grant"));
		flag_addGrant = true;
		refresh();
	}
	
	/**
	 * Unset add grant flag
	 */
	public void unsetAddGrant() {
		flag_addGrant = false;
		refresh();
	}
	
	/**
	 * Sets get grants flag
	 */
	public void setGetGrants() {
		msg.setHTML(GeneralComunicator.i18nExtension("sample.tab.get.grants"));
		flag_getGrants = true;
		refresh();
	}
	
	/**
	 * Unset add grant flag
	 */
	public void unsetGetGrants() {
		flag_getGrants = false;
		refresh();
	}
	
	/**
	 * Sets revoke grants flag
	 */
	public void setRevokeGrant() {
		msg.setHTML(GeneralComunicator.i18nExtension("sample.tab.revoke.grant"));
		flag_revokeGrant = true;
		refresh();
	}
	
	/**
	 * Unset revoke grant flag
	 */
	public void unsetRevokeGrant() {
		flag_revokeGrant = false;
		refresh();
	}
}
